// ----
// ---- file   : sample_utils.cpp
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2014 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : 
// ----
// ---- created: 25Oct2009
// ---- changed: 29Apr2010, 11May2010, 25Feb2011, 04May2013, 08May2013, 12Feb2014
// ----          
// ----
// ----

#include "tksampleedit.h"


void tksampleedit_ringbuf_copy_samples(YAC_Object *_dst, sUI _dstNumCh, sUI _dstChOff, sUI _dstFrameOff, 
                                       sUI _numFrames, sUI _numCh, 
                                       YAC_Object *_src, sUI _srcNumCh, sUI _srcChOff, sUI _srcFrameOff
                                       ) {
   if(YAC_Is_FloatArray(_dst))
   {
      if(YAC_Is_FloatArray(_src))
      {
         YAC_CAST_ARG(YAC_FloatArray, src, _src);
         YAC_CAST_ARG(YAC_FloatArray, dst, _dst);

         sF32 *d = dst->elements + _dstChOff;
         sF32 *s = src->elements + _srcChOff;

         sUI srcSmpOff = _srcFrameOff * _srcNumCh;
         sUI dstSmpOff = _dstFrameOff * _dstNumCh;
         sUI i;

         for(i=0; i<_numFrames; i++)
         {
            if(dstSmpOff >= dst->num_elements)
            {
               dstSmpOff -= dst->num_elements;
            }

            if(srcSmpOff >= src->num_elements)
            {
               srcSmpOff -= src->num_elements;
            }

            for(sUI chIdx = 0; chIdx < _numCh; chIdx++)
            {
               d[dstSmpOff+chIdx] = s[srcSmpOff+chIdx];
            }

            dstSmpOff += _dstNumCh;
            srcSmpOff += _srcNumCh;
         }
      }
   }
}


sF32 YAC_CALL tksampleedit_buf_get_max(YAC_Object *_src, sUI _numFrames, sUI _numCh, sUI _chOff) {
   sF32 r = 0.0f;

   if(YAC_Is_FloatArray(_src))
   {
      YAC_CAST_ARG(YAC_FloatArray, src, _src);

      sF32 *s = src->elements;

      sUI i;
      sUI off =  _chOff;
      for(i=0; i<_numFrames; i++)
      {
         sF32 f = s[off];

         if(f < 0)
            f = -f;

         if(f > r)
            r = f;

         off += _numCh;
      }
   }

   return r;
}

void YAC_CALL tksampleedit_mix_stereo_input_buf(YAC_Object *_faOutput,
                                                YAC_Object *_faInput,
                                                YAC_Object *_faMixerLevels,
                                                sUI _numInputChannels,
                                                sUI _numOutputChannels,
                                                sUI _numFrames,
                                                sUI _outChOff,
                                                sUI _inChOff
                                                ) {
   if(_numInputChannels > 0)
   {
      if(YAC_Is_FloatArray(_faOutput))
      {
         YAC_CAST_ARG(YAC_FloatArray, output, _faOutput);
         
         if(YAC_Is_FloatArray(_faInput))
         {
            YAC_CAST_ARG(YAC_FloatArray, input, _faInput);
            
            if(YAC_Is_FloatArray(_faMixerLevels))
            {
               YAC_CAST_ARG(YAC_FloatArray, mixerLevels, _faMixerLevels);

               sUI ooff = _outChOff;
               sUI ioff = _inChOff;

               for(sUI frameNr=0; frameNr<_numFrames; frameNr++)
               {
                  sF32 l = input->elements[ioff + 0] * mixerLevels->elements[_inChOff + 0];
                  sF32 r = input->elements[ioff + 1] * mixerLevels->elements[_inChOff + 1];

                  output->elements[ooff + 0] = l;
                  output->elements[ooff + 1] = r;

                  // Next frame
                  ooff += _numOutputChannels; 
                  ioff += _numInputChannels;
               }
            }
         }
      }
   }
}


void YAC_CALL tksampleedit_mixclip_stereo_mixbuf(YAC_Object *_faOutput,
                                                 YAC_Object *_faMixBuf,
                                                 sF32 _levelL,
                                                 sF32 _levelR,
                                                 sF32 _clipLevel,
                                                 sUI _numOutputChannels,
                                                 sUI _numFrames,
                                                 sUI _outChOff
                                                 ) {

   if(YAC_Is_FloatArray(_faOutput))
   {
      YAC_CAST_ARG(YAC_FloatArray, output, _faOutput);

      if(YAC_Is_FloatArray(_faMixBuf))
      {
         YAC_CAST_ARG(YAC_FloatArray, mixBuf, _faMixBuf);

         sUI inOff = 0;

         for(sUI frameNr=0; frameNr<_numFrames; frameNr++)
         {
            sF32 l = output->elements[_outChOff + 0] + mixBuf->elements[inOff + 0] * _levelL;
            if(l < -_clipLevel)
            {
               l = -_clipLevel;
            }
            else if(l > _clipLevel)
            {
               l = _clipLevel;
            }
            output->elements[_outChOff + 0] = l;


            sF32 r = output->elements[_outChOff + 1] + mixBuf->elements[inOff + 1] * _levelR;
            if(r < -_clipLevel)
            {
               r = -_clipLevel;
            }
            else if(r > _clipLevel)
            {
               r = _clipLevel;
            }
            output->elements[_outChOff + 1] = r;

            // Next frame
            _outChOff += _numOutputChannels;
            inOff += 2;
         }
         
      }
   }
}
