// ----
// ---- file   : tksampleedit.tks
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- legal  : (c) 2009-2014 by Bastian Spiegel. 
// ----          Distributed under terms of the GNU LESSER GENERAL PUBLIC LICENSE (LGPL). See 
// ----          http://www.gnu.org/licenses/licenses.html#LGPL or COPYING for further information.
// ----
// ---- info   : 
// ----
// ---- created: 25Oct2009
// ---- changed: 11May2010, 20Jul2010, 25Feb2011, 04May2013, 12Feb2014
// ----          
// ----
// ----

#ifndef __TKSAMPLEEDIT_H__
#define __TKSAMPLEEDIT_H__


#include <math.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define YAC_PRINTF defined
#define YAC_BIGSTRING
#include <yac.h>

#include "tks-opengl.h"


YG("tksampleedit");


extern void tksampleedit_rms_init(void);


/* @function tksampleedit_render_sample,FloatArray samples,int numCh,int sampleOffset,int numSamples,float w,float h,int renderStyle,int interpolStyle
Render a floating point waveform

@arg samples
@arg numCh
@arg frameOffset
@arg numFrames
@arg w
@arg h
@arg renderStyle
@arg interpolStyle
*/
YF void YAC_CALL tksampleedit_render_sample(YAC_Object *_data, sUI _numCh, sUI _frameOffset, sUI _numFrames, sF32 _w, sF32 _h, sF32 _min, sF32 _max, sSI _renderStyle, sSI _interpolStyle);


/* @function tksampleedit_render_rms,FloatArray samples,int numCh,int sampleOffset,int numSamples,float w,float h
Render root mean square (RMS) of a floating point waveform 

*/
YF void YAC_CALL tksampleedit_render_rms(YAC_Object *_data, sUI _numCh, sUI _frameOffset, sUI _numFrames, sF32 _w, sF32 _h, sF32 _min, sF32 _max);


/* @function tksampleedit_calc_rms,FloatArray samples,int numCh:float
Calculate root mean square (RMS) of sample array.

@arg samples
@arg numCh
@return Root mean square
*/
YF sF32 YAC_CALL tksampleedit_calc_rms(YAC_Object *_fa, sUI _numCh);


YF void YAC_CALL tksampleedit_ringbuf_copy_samples(YAC_Object *_dst, sUI _dstNumCh, sUI _dstChOff, sUI _dstFrameOff, sUI _numFrames, sUI _numCh, YAC_Object *_src, sUI _srcNumCh, sUI _srcChOff, sUI _srcFrameOff);


YF sF32 YAC_CALL tksampleedit_buf_get_max(YAC_Object *_src, sUI _numFrames, sUI _numCh, sUI _chOff);

// used by Audio.FillStreamBuffer()
YF void YAC_CALL tksampleedit_mix_stereo_input_buf(YAC_Object *_faOutput, YAC_Object *_faInput, YAC_Object *_faMixerLevels, sUI _numInputChannels, sUI _numOutputChannels, sUI _numFrames, sUI _outChOff, sUI _inChOff);

// used by Audio.ProcessSampleFragment()
YF void YAC_CALL tksampleedit_mixclip_stereo_mixbuf(YAC_Object *_faOutput, YAC_Object *_faMixBuf, sF32 _levelL, sF32 _levelR, sF32 _clipLevel, sUI _numOutputChannels, sUI _numFrames, sUI _outChOff);


#endif // __TKSAMPLEEDIT_H__
